from traceback import format_exc
import os
import sys
import argparse
from llmengine.m2v3bgererank import M2V3BgeRerank
from llmengine.base_bgererank import get_rerank_class
from typing import List, Dict
from appPublic.registerfunction import RegisterFunction
from appPublic.worker import awaitify
from appPublic.log import debug, exception
from ahserver.serverenv import ServerEnv
from ahserver.globalEnv import stream_response
from ahserver.webapp import webserver

helptext = """BGE Reranker API:

1. Rerank Endpoint:
path: /v1/bgererank
method: POST
headers: {
    "Content-Type": "application/json"
}
data: {
    "query": "苹果公司在北京开设新店",
    "documents": [
        "苹果公司在上海开设新店",
        "苹果公司在北京发布新产品"
    ],
    "top_k": 5
}
response: {
    "object": "list",
    "data": [
        {
            "text": "苹果公司在北京发布新产品",
            "rerank_score": 0.95
        },
        ...
    ]
}

2. Docs Endpoint:
path: /v1/docs
method: GET
response: This help text
"""

def init():
    rf = RegisterFunction()
    rf.register('bgererank', rerank)
    rf.register('docs', docs)
    debug("注册路由: bgererank, docs")

async def docs(request, params_kw, *params, **kw):
    return helptext

async def rerank(request, params_kw, *params, **kw):
    debug(f'{params_kw=}')
    se = ServerEnv()
    engine = se.engine
    query = params_kw.get('query')
    documents = params_kw.get('documents', [])
    top_k = params_kw.get('top_k', 10)
    if query is None:
        e = exception(f'query is None')
        raise e
    if not isinstance(documents, list) or not all(isinstance(doc, str) for doc in documents):
        e = exception(f'documents must be a list of strings')
        raise e
    try:
        reranked = await engine.rerank(query, documents, top_k)
        debug(f'{reranked=}')
        return {
            "object": "list",
            "data": reranked
        }
    except Exception as e:
        exception(f'重排序失败: {str(e)}')
        raise

def main():
    parser = argparse.ArgumentParser(prog="BGE Reranker Service")
    parser.add_argument('-w', '--workdir')
    parser.add_argument('-p', '--port')
    parser.add_argument('model_path')
    args = parser.parse_args()
    Klass = get_rerank_class(args.model_path)
    if Klass is None:
        e = Exception(f'{args.model_path} has not mapping to a model class')
        exception(f'{e}, {format_exc()}')
        raise e
    se = ServerEnv()
    se.engine = Klass(args.model_path)
    workdir = args.workdir or os.getcwd()
    port = args.port
    debug(f'{args=}')
    webserver(init, workdir, port)

if __name__ == '__main__':
    main()