from typing import List, Dict
from appPublic.log import debug, info, error
from appPublic.worker import awaitify
from llmengine.base_bgererank import BaseRerank, rerank_register
from pymilvus.model.reranker import BGERerankFunction
import torch
import asyncio

class M2V3BgeRerank(BaseRerank):
    def __init__(self, model_id: str):
        """Initialize BGE Reranker with GPU support and FP16."""
        try:
            self.model_id = model_id
            self.model_name = model_id.split('/')[-1]
            # Load BGE Reranker with cuda:0 and FP16
            self.reranker = BGERerankFunction(
                model_name=model_id,
                device="cuda:7" if torch.cuda.is_available() else "cpu",
                use_fp16=True
            )
            info(f"BGE Reranker 初始化成功，模型路径: {model_id}, 设备: {'cuda:0' if torch.cuda.is_available() else 'cpu'}, FP16: True")
        except Exception as e:
            error(f"BGE Reranker 初始化失败: {str(e)}")
            raise

    async def rerank(self, query: str, documents: List[str], top_k: int = 10) -> List[Dict]:
        """
        使用 BGE Reranker 对文档进行重排序，仅接收 text 列表，返回 text 和 rerank_score。

        参数:
            query (str): 查询文本
            documents (List[str]): 仅包含文本的文档列表
            top_k (int): 返回的最大结果数，默认为 10

        返回:
            List[Dict]: 重排序后的文档列表，包含 text 和 rerank_score
        """
        try:
            if not query:
                raise ValueError("查询文本不能为空")
            if not documents:
                info("无文档可重排序")
                return []

            debug(f"重排序输入: query={query[:50]}..., 文档数量={len(documents)}")

            # 定义同步 rerank 函数
            def sync_rerank(query, texts, top_k):
                return self.reranker(query=query, documents=texts, top_k=min(top_k, len(texts)))

            # 使用 run_in_executor 运行同步 rerank
            loop = asyncio.get_event_loop()
            rerank_results = await loop.run_in_executor(None, lambda: sync_rerank(query, documents, top_k))

            # 构建重排序结果，仅包含 text 和 rerank_score
            reranked = [
                {
                    "text": result.text,
                    "rerank_score": result.score
                }
                for result in rerank_results
            ]
            info(f"重排序返回 {len(reranked)} 条结果")
            return reranked

        except Exception as e:
            error(f"重排序失败: {str(e)}")
            raise

rerank_register('bge-reranker-v2-m3', M2V3BgeRerank)