import os
import torch
import re
import traceback
from transformers import AutoModelForSeq2SeqLM, AutoTokenizer
from appPublic.log import debug, error, warning, info
from appPublic.worker import awaitify
from base_triple import BaseTripleExtractor, llm_register

class MRebelTripleExtractor(BaseTripleExtractor):
    def __init__(self, model_path: str):
        super().__init__(model_path)
        try:
            debug(f"Loading tokenizer from {model_path}")
            self.tokenizer = AutoTokenizer.from_pretrained(model_path)
            debug(f"Loading model from {model_path}")
            self.model = AutoModelForSeq2SeqLM.from_pretrained(model_path)
            self.device = self.use_mps_if_possible()
            self.triplet_id = self.tokenizer.convert_tokens_to_ids("<triplet>")
            debug(f"Loaded mREBEL model, triplet_id: {self.triplet_id}")
        except Exception as e:
            error(f"Failed to load mREBEL model: {str(e)}")
            raise RuntimeError(f"Failed to load mREBEL model: {str(e)}")

        self.gen_kwargs = {
            "max_length": 512,
            "min_length": 10,
            "length_penalty": 0.5,
            "num_beams": 3,
            "num_return_sequences": 1,
            "no_repeat_ngram_size": 2,
            "early_stopping": True,
            "decoder_start_token_id": self.triplet_id,
        }

    def split_document(self, text: str, max_chunk_size: int = 150) -> list:
        """Split document into semantic chunks."""
        sentences = re.split(r'(?<=[。！？；\n])', text)
        chunks = []
        current_chunk = ""
        for sentence in sentences:
            if len(current_chunk) + len(sentence) <= max_chunk_size:
                current_chunk += sentence
            else:
                if current_chunk:
                    chunks.append(current_chunk)
                current_chunk = sentence
        if current_chunk:
            chunks.append(current_chunk)
        debug(f"Text split into: {len(chunks)} chunks")
        return chunks

    def extract_triplets_typed(self, text: str) -> list:
        """Parse mREBEL generated text for triplets."""
        triplets = []
        debug(f"Raw generated text: {text}")

        tokens = []
        in_tag = False
        buffer = ""
        for char in text:
            if char == '<':
                in_tag = True
                if buffer:
                    tokens.append(buffer.strip())
                    buffer = ""
                buffer += char
            elif char == '>':
                in_tag = False
                buffer += char
                tokens.append(buffer.strip())
                buffer = ""
            else:
                buffer += char
        if buffer:
            tokens.append(buffer.strip())

        special_tokens = ["<s>", "<pad>", "</s>", "tp_XX", "__en__", "__zh__", "zh_CN"]
        tokens = [t for t in tokens if t not in special_tokens and t]
        debug(f"Processed tokens: {tokens}")

        i = 0
        while i < len(tokens):
            if tokens[i] == "<triplet>" and i + 5 < len(tokens):
                entity1 = tokens[i + 1]
                type1 = tokens[i + 2][1:-1] if tokens[i + 2].startswith("<") and tokens[i + 2].endswith(">") else ""
                entity2 = tokens[i + 3]
                type2 = tokens[i + 4][1:-1] if tokens[i + 4].startswith("<") and tokens[i + 4].endswith(">") else ""
                relation = tokens[i + 5]

                if entity1 and type1 and entity2 and type2 and relation:
                    triplets.append({
                        'head': entity1.strip(),
                        'head_type': type1,
                        'type': relation.strip(),
                        'tail': entity2.strip(),
                        'tail_type': type2
                    })
                    debug(f"Added triplet: {entity1}({type1}) - {relation} - {entity2}({type2})")
                i += 6
            else:
                i += 1

        return triplets

    async def extract_triplets(self, text: str) -> list:
        """Extract triplets from text and return unique triplets."""
        try:
            if not text:
                raise ValueError("Text cannot be empty")

            text_chunks = self.split_document(text, max_chunk_size=150)
            debug(f"Text split into {len(text_chunks)} chunks")

            all_triplets = []
            for i, chunk in enumerate(text_chunks):
                debug(f"Processing chunk {i + 1}/{len(text_chunks)}: {chunk[:50]}...")

                model_inputs = self.tokenizer(
                    chunk,
                    max_length=256,
                    padding=True,
                    truncation=True,
                    return_tensors="pt"
                ).to(self.device)

                try:
                    generated_tokens = self.model.generate(
                        model_inputs["input_ids"],
                        attention_mask=model_inputs["attention_mask"],
                        **self.gen_kwargs
                    )
                    decoded_preds = self.tokenizer.batch_decode(generated_tokens, skip_special_tokens=False)
                    for idx, sentence in enumerate(decoded_preds):
                        debug(f"Chunk {i + 1} generated text: {sentence}")
                        triplets = self.extract_triplets_typed(sentence)
                        if triplets:
                            debug(f"Chunk {i + 1} extracted {len(triplets)} triplets")
                            all_triplets.extend(triplets)
                except Exception as e:
                    warning(f"Error processing chunk {i + 1}: {str(e)}")
                    continue

            unique_triplets = []
            seen = set()
            for t in all_triplets:
                identifier = (t['head'].lower(), t['type'].lower(), t['tail'].lower())
                if identifier not in seen:
                    seen.add(identifier)
                    unique_triplets.append(t)

            info(f"Extracted {len(unique_triplets)} unique triplets")
            return unique_triplets

        except Exception as e:
            error(f"Failed to extract triplets: {str(e)}")
            debug(f"Traceback: {traceback.format_exc()}")
            return []

llm_register("mrebel-large", MRebelTripleExtractor)